package check_mail_address_exchange;

use strict;

use Net::LDAP;
use Net::LDAP::Util qw(ldap_error_text ldap_error_name ldap_error_desc);

use vars qw(@ISA @EXPORT @EXPORT_OK $VERSION);

use Exporter;

$VERSION = 0.01;

@ISA = qw(Exporter);

#    ( :DEFAULT)
@EXPORT      = qw(verify_address_exchange);

# ,   
@EXPORT_OK   = qw(verify_address_exchange);

######################################################################

use check_mail_address_common;

sub verify_address_exchange {
    my($address, $servers_list, $dn, $password, $base, $syslog_param, $verbose) = @_;

    my($command, $answer);

    $address =~ s/^\s*\<//;
    $address =~ s/\>\s*$//;
    my($mailbox, $domain) = ($address =~ /\s*(.+)\@(.+)\s*$/);
    my(@servers) = split(/[,: ]+/, $servers_list);

    my($use_syslog) = 1 if ($syslog_param);

&check_mail_address_common::print_syslog($mailbox, $domain, "",
"test! $use_syslog", "",
"check passed", $use_syslog, $syslog_param);

    my($filter) = "(proxyAddresses=smtp:$address)";
    my($attrs) = ['proxyAddresses'];

    if ($verbose) {
        use IO::Handle;
	STDOUT->autoflush(1);
    }
    foreach my $host (@servers) {
	$command = "connect to LDAP server '$host'\n";
        &check_mail_address_common::print_msg("connect to LDAP server '$host'...", $verbose);
	my($ldap) = Net::LDAP->new($host);
        unless ($ldap) {
	    &check_mail_address_common::print_msg(" failed...\n", $verbose);
	    $answer = "431 connection to LDAP server failed\n";
	    &check_mail_address_common::print_syslog($mailbox, $domain, $host, "connect to LDAP server $host", "couldn't connect to LDAP server $host", "check passed", $use_syslog, $syslog_param);
        } else {
	    &check_mail_address_common::print_msg(" ok...\n", $verbose);
	    $command = "bind (authenticate) to LDAP server '$host'\n";
    	    &check_mail_address_common::print_msg("try to bind (authenticate)...", $verbose);
	    my($mesg) = $ldap->bind(
    		dn		=> $dn,
    	        password	=> $password,
    		version 	=> 3
	    );
	    if ($mesg->is_error) {
		&check_mail_address_common::print_msg(" failed...\n", $verbose);
		my($ldap_error) = get_ldap_error_text($mesg->code);
	        &check_mail_address_common::print_msg("$ldap_error\n", $verbose);
		$answer = "431 binding to LDAP server failed\n";
		&check_mail_address_common::print_syslog($mailbox, $domain, $host,
		    "bind to LDAP server $host", $ldap_error,
		    , "check passed", $use_syslog, $syslog_param);
	    } else {
		&check_mail_address_common::print_msg(" ok...\n", $verbose);
		$command = "verify address on LDAP server '$host'\n";
	        &check_mail_address_common::print_msg("search \"$filter\"...", $verbose);
    		$mesg = $ldap->search(
	    	    base	=> $base,
		    filter	=> $filter,
		    attrs	=> $attrs
		);

		if ($mesg->is_error) {
		    $answer = "431 searching on LDAP server failed\n";
    	    	    &check_mail_address_common::print_msg(" failed...\n", $verbose);
		    my($ldap_error) = get_ldap_error_text($mesg->code);
	    	    &check_mail_address_common::print_msg("$ldap_error\n", $verbose);
		    &check_mail_address_common::print_syslog($mailbox, $domain, $host,
			"verify address '$address' on LDAP server $host", $ldap_error,
			"check passed", $use_syslog, $syslog_param);
	        } else {
		    &check_mail_address_common::print_msg(" ok\n", $verbose);
		    &check_mail_address_common::print_msg($mesg->all_entries." entries found\n", $verbose);
#
#printf "count=%d\n", $mesg->count();
#printf "error=%s\n", $mesg->error;
		    if ($verbose) {
	    	        my($entry);
    	    		foreach $entry ($mesg->all_entries) {
			    $entry->dump;
			}
		    }
		    &check_mail_address_common::print_msg("unbind\n", $verbose);
		    $ldap->unbind;
		    if ($mesg->all_entries) {
			$answer = "250 Recipient Ok\n";
			&check_mail_address_common::print_syslog($mailbox, $domain, $host, "verify address '$address' on LDAP server $host", "250 Recipient Ok", "check passed", $use_syslog, $syslog_param);
		    } else {
			$answer = "550 5.1.1 User unknown\n";
			&check_mail_address_common::print_syslog($mailbox, $domain, $host, "verify address '$address' on LDAP server $host", "550 5.1.1 User unknown", "user unknown", $use_syslog, $syslog_param);
		    }
		    return(($command, $answer));
		}
	    }
	    &check_mail_address_common::print_msg("unbind\n", $verbose);
	    $ldap->unbind;
	}
    }
    return(($command, $answer));
}

sub get_ldap_error_text() {
    my($code) = @_;
    return("error $code (".ldap_error_name($code).") ".ldap_error_text($code)." ".ldap_error_desc($code));
}

1;
